VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DebuggerClass"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' John Robbins
' Microsoft Systems Journal - August, 1997
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FILE          :   DebuggerClass.cls
' DESCRIPTION   :
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Class private variables
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The required full filename of the debuggee.
Private m_szDebuggee As String
' The optional command line to use.
Private m_szCmdLine As String
' The optional startup directory.
Private m_szWorkDir As String

' The executive class that the debugger will use.
Private m_clsExecutive As BaseExecutive

' The synchronization class that the debugger will use.  This is its own
'  instance.
Private m_clsSynch As DebugSynchClass

' The process ID for the debuggee.
Private m_dwDebuggeePID As Long

' The error code returned when I tried to create the debuggee.
Private m_dwCreateError As Long

' The individual debug events.  These are declared as private members
'  and all have specific friend property get functions that will return
'  them.  If the debugger class is supposed to move into an ActiveX
'  control, then this decision will have to be rethought.
Private m_CreateProcessDbgEvt As CREATE_PROCESS_DEBUG_INFO
Private m_CreateThreadDbgEvt As CREATE_THREAD_DEBUG_INFO
Private m_ExceptionDbgEvt As EXCEPTION_DEBUG_INFO
Private m_ExitProcessDbgEvt As EXIT_PROCESS_DEBUG_INFO
Private m_ExitThreadDbgEvt As EXIT_THREAD_DEBUG_INFO
Private m_ODSDbgEvt As OUTPUT_DEBUG_STRING_INFO
Private m_LoadDllDbgEvt As LOAD_DLL_DEBUG_INFO
Private m_UnloadDllDbgEvt As UNLOAD_DLL_DEBUG_INFO
Private m_RipInfo As RIP_INFO

Friend Property Set clsBaseExecutive(clsBase As BaseExecutive)
   Set m_clsExecutive = clsBase
End Property

Friend Property Get CreateProcessDbgEvt() As CREATE_PROCESS_DEBUG_INFO
   CreateProcessDbgEvt = m_CreateProcessDbgEvt
End Property

Friend Property Get CreateThreadDbgEvt() As CREATE_THREAD_DEBUG_INFO
   CreateThreadDbgEvt = m_CreateThreadDbgEvt
End Property

Friend Property Get ExceptionDbgEvt() As EXCEPTION_DEBUG_INFO
   ExceptionDbgEvt = m_ExceptionDbgEvt
End Property

Friend Property Get ExitProcessDbgEvt() As EXIT_PROCESS_DEBUG_INFO
   ExitProcessDbgEvt = m_ExitProcessDbgEvt
End Property

Friend Property Get ExitThreadDbgEvt() As EXIT_THREAD_DEBUG_INFO
   ExitThreadDbgEvt = m_ExitThreadDbgEvt
End Property

Friend Property Get ODSDbgEvt() As OUTPUT_DEBUG_STRING_INFO
   ODSDbgEvt = m_ODSDbgEvt
End Property

Friend Property Get LoadDllDbgEvt() As LOAD_DLL_DEBUG_INFO
   LoadDllDbgEvt = m_LoadDllDbgEvt
End Property

Friend Property Get UnloadDllDbgEvt() As UNLOAD_DLL_DEBUG_INFO
   UnloadDllDbgEvt = m_UnloadDllDbgEvt
End Property

Friend Property Get RipInfo() As RIP_INFO
   RipInfo = m_RipInfo
End Property


Friend Property Get dwDebuggeePID() As Long
   dwDebuggeePID = m_dwDebuggeePID
End Property

Friend Property Get dwCreateError() As Long
   dwCreateError = m_dwCreateError
End Property


''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : Class_Initialize
' DISCUSSION      :
'  The initialization for the class.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub Class_Initialize()
   Set m_clsExecutive = Nothing
   Set m_clsSynch = New DebugSynchClass
   m_dwDebuggeePID = 0
End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : Class_Terminate
' DISCUSSION      :
'  The termination for the class.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Private Sub Class_Terminate()
   Set m_clsExecutive = Nothing
   Set m_clsSynch = Nothing

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : SetDebuggeeInfo
' DISCUSSION      :
'  Sets the information that MUST be set before the class can be used.
' PARAMETERS      :
'  szDebuggee  - The full path and name of the program to debug.
'  szCmdLine   - The command line for the debuggee.  This can be
'                vbNullString.
'  szWorkDir   - The working directory for the debuggee.  This can be
'                vbNullString.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub SetDebuggeeInfo(szDebuggee As String, _
                          Optional szCmdLine As String = vbNullString, _
                          Optional szWorkDir As String = vbNullString)

   ' Set the variables.
   m_szDebuggee = szDebuggee
   m_szCmdLine = szCmdLine
   m_szWorkDir = szWorkDir

End Sub

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : StartDebuggee
' DISCUSSION      :
'  Starts up the debuggee.  This can only be called from the debug
'  thread.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function StartDebuggee() As Boolean

   On Error GoTo StartDebuggee_Error

   Dim bRet As Long
   Dim si As STARTUPINFO
   Dim pi As PROCESS_INFORMATION

   ' Initialize si and pi to known values.
   si.cb = &H44
   si.cbReserved2 = 0
   si.dwFillAttribute = 0
   si.dwFlags = 0
   si.dwX = 0
   si.dwX = 0
   si.dwY = 0
   si.dwXSize = 0
   si.dwYSize = 0
   si.dwXCountChars = 0
   si.dwYCountChars = 0
   si.dwFillAttribute = 0
   si.dwFlags = 0
   si.wShowWindow = 0
   si.cbReserved2 = 0
   si.lpReserved2 = 0
   si.hStdInput = 0
   si.hStdOutput = 0
   si.hStdError = 0

   pi.hProcess = 0
   pi.hThread = 0
   pi.dwProcessID = 0
   pi.dwThreadID = 0

   ' Try and start up the application to debug.
   bRet = CreateProcess(m_szDebuggee, _
                        m_szCmdLine, _
                        0, _
                        0, _
                        0, _
                        DEBUG_ONLY_THIS_PROCESS, _
                        vbNullString, _
                        m_szWorkDir, _
                        si, _
                        pi)

   ' Set the create error.
   m_dwCreateError = GetLastError

   ' If the CreateProcesses failed, signal the event that the debuggee
   '  failed to start, and then leave.
   If (0 = bRet) Then
      m_clsSynch.SignalBadStartup
      StartDebuggee = False
      Exit Function
   End If

   ' Close some handles I don't need.
   bRet = CloseHandle(pi.hProcess)
#If DEBUGBUILD Then
   If (0 = bRet) Then
      MsgBox ("StartDebuggee CloseHandle failed!")
   End If
#End If

   bRet = CloseHandle(pi.hThread)
#If DEBUGBUILD Then
   If (0 = bRet) Then
      MsgBox ("StartDebuggee CloseHandle failed!")
   End If
#End If

   ' Set the unique PID for the debuggee.
   m_dwDebuggeePID = pi.dwProcessID

   m_clsSynch.dwUniqueID = m_dwDebuggeePID

   ' Create the synchronization objects for the synch class for just
   '  this thread.
   m_clsSynch.CreateSynchObjects

   ' At least the debuggee was able to start so signal that event.
   m_clsSynch.SignalGoodStartup

   StartDebuggee = True

   Exit Function

StartDebuggee_Error:
   MsgBox ("Got an error in StartDebuggee: " + Err.Description)

End Function

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FUNCTION        : ProcessDebugEvents
' DISCUSSION      :
'  Does the looping and processing of debug events.
' PARAMETERS      :
'  None.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub ProcessDebugEvents()

   Dim DbgEvt As DEBUG_EVENT

   Dim dwContType As Long

   Dim bRet As Long

   Dim bDebugRunning

   Dim iObject As Long

   ' The debugger loop is always running.
   bDebugRunning = True

   While (True = bDebugRunning)

      ' Which thing am I supposed to be doing?
      iObject = m_clsSynch.WaitForSynchObject(INFINITE)

      Select Case iObject

         ' Quitting.
         Case 0
            bDebugRunning = False

         ' Pausing.
         Case 1
            m_clsExecutive.PauseProcess

         ' Resuming.
         Case 2
            m_clsExecutive.ResumeProcess

         ' Regular debugging.
         Case 3
            ' Wait a bit for the debug event.
            If (1 = WaitForDebugEvent(DbgEvt, 100)) Then

               ' Default to continuing.
               dwContType = DBG_CONTINUE

               ' Process all the debug events by calling the executive
               '  class that I was passed at startup.
               Select Case DbgEvt.dwDebugEventCode

                  Case CREATE_PROCESS_DEBUG_EVENT
                     LSet m_CreateProcessDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugCreateProcess Me, _
                                                       DbgEvt.dwProcessID, _
                                                       DbgEvt.dwThreadID

                  Case CREATE_THREAD_DEBUG_EVENT
                     LSet m_CreateThreadDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugCreateThread Me, _
                                                      DbgEvt.dwProcessID, _
                                                      DbgEvt.dwThreadID

                  Case EXIT_PROCESS_DEBUG_EVENT
                     LSet m_ExitProcessDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugExitProcess Me, _
                                                     DbgEvt.dwProcessID, _
                                                     DbgEvt.dwThreadID
                     ' EXIT_PROCESS_DEBUG_EVENT is handled special in
                     '  that I will just go ahead and exit the
                     '  processing subroutine.
                     Exit Sub

                  Case EXIT_THREAD_DEBUG_EVENT
                     LSet m_ExitThreadDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugExitThread Me, _
                                                    DbgEvt.dwProcessID, _
                                                    DbgEvt.dwThreadID

                  Case EXCEPTION_DEBUG_EVENT
                     LSet m_ExceptionDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugException Me, _
                                                   dwContType, _
                                                   DbgEvt.dwProcessID, _
                                                   DbgEvt.dwThreadID

                  Case OUTPUT_DEBUG_STRING_EVENT
                     LSet m_ODSDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugODS Me, _
                                             DbgEvt.dwProcessID, _
                                             DbgEvt.dwThreadID

                  Case LOAD_DLL_DEBUG_EVENT
                     LSet m_LoadDllDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugDllLoad Me, _
                                                 DbgEvt.dwProcessID, _
                                                 DbgEvt.dwThreadID

                  Case UNLOAD_DLL_DEBUG_EVENT
                     LSet m_UnloadDllDbgEvt = DbgEvt.dwUnionData
                     m_clsExecutive.DebugDllUnload Me, _
                                                   DbgEvt.dwProcessID, _
                                                   DbgEvt.dwThreadID

                  Case RIP_EVENT
                     LSet m_RipInfo = DbgEvt.dwUnionData
                     m_clsExecutive.DebugRipInfo Me, _
                                                 DbgEvt.dwProcessID, _
                                                 DbgEvt.dwThreadID

               End Select

               bRet = ContinueDebugEvent(DbgEvt.dwProcessID, _
                                         DbgEvt.dwThreadID, _
                                         dwContType)
            End If

         Case Else
#If DEBUGBUILD Then
            MsgBox ("WaitForSynchObject returned something other " & _
                    "than 0 - 3!!")
#End If

      End Select
   Wend

End Sub

